#include "nan.h"

#include "onig-reg-exp.h"
#include "onig-result.h"

using ::v8::Exception;
using ::v8::String;

OnigRegExp::OnigRegExp(const string& source)
    : source_(source),
      regex_(NULL) {
  OnigErrorInfo error;
  const UChar* sourceData = (const UChar*)source.data();
  int status = onig_new(&regex_, sourceData, sourceData + source.length(),
                        ONIG_OPTION_CAPTURE_GROUP, ONIG_ENCODING_UTF8,
                        ONIG_SYNTAX_DEFAULT, &error);

  if (status != ONIG_NORMAL) {
    UChar errorString[ONIG_MAX_ERROR_MESSAGE_LEN];
    onig_error_code_to_str(errorString, status, &error);
    ThrowException(Exception::Error(String::New(reinterpret_cast<char*>(errorString))));
  }
}

OnigRegExp::~OnigRegExp() {
  if (regex_) onig_free(regex_);
}

bool OnigRegExp::Contains(const string& value) {
  return source_.find(value) != string::npos;
}

shared_ptr<OnigResult> OnigRegExp::Search(const string& searchString,
                                          size_t position) {
  if (!regex_) {
    ThrowException(Exception::Error(String::New("RegExp is not valid")));
    return shared_ptr<OnigResult>();
  }

  int end = searchString.size();
  OnigRegion* region = onig_region_new();
  const UChar* searchData = (const UChar*)searchString.data();
  int status = onig_search(regex_, searchData, searchData + end,
                           searchData + position, searchData + end, region,
                           ONIG_OPTION_NONE);

  if (status != ONIG_MISMATCH) {
    return shared_ptr<OnigResult>(new OnigResult(region));
  } else {
    onig_region_free(region, 1);
    return shared_ptr<OnigResult>();
  }
}
